// Size.cxx

#include "testap3.h"
/* Macro to restrict a given value to an upper or lower boundary value */
#define BOUND(x,min,max) ((x) < (min) ? (min) : ((x) > (max) ? (max) : (x)))

SIZE  g_wm_size;  // store current WM_SIZE size
static INT iSem = 0;

VOID GetRealClientRect ( HWND hwnd, PRECT lprc)
{
   DWORD_PTR dwStyle;

   dwStyle = GetWindowLongPtr (hwnd, GWL_STYLE);
   GetClientRect (hwnd,lprc);

   if (dwStyle & WS_HSCROLL)
      lprc->bottom += GetSystemMetrics (SM_CYHSCROLL);

   if (dwStyle & WS_VSCROLL)
      lprc->right  += GetSystemMetrics (SM_CXVSCROLL);

}

VOID SetScrollRanges(HWND hwnd, WORD wid, WORD ht)
{
   RECT       rc;
   INT        iRangeH, iRangeV, i;

   if(!iSem) {   // no re-entry
      iSem++;

      GetRealClientRect (hwnd, &rc);

      if( m_FlagStretch ) {
         SetScrollRange (hwnd, SB_VERT, 0, 0, TRUE);
         SetScrollRange (hwnd, SB_HORZ, 0, 0, TRUE);
         goto Exit_Scroll;
      }

      for (i = 0; i < 2; i++) {
         iRangeV = m_sDIBSize.cy - rc.bottom;
         iRangeH = m_sDIBSize.cx - rc.right;

         if (iRangeH < 0) iRangeH = 0;
         if (iRangeV < 0) iRangeV = 0;

         if(( GetScrollPos ( hwnd, SB_VERT) > iRangeV ) ||
            ( GetScrollPos ( hwnd, SB_HORZ) > iRangeH ) )
         {
            // set a repaint
            InvalidateRect (hwnd, NULL, TRUE);
         }

         SetScrollRange (hwnd, SB_VERT, 0, iRangeV, TRUE);
         SetScrollRange (hwnd, SB_HORZ, 0, iRangeH, TRUE);

         GetClientRect (hwnd, &rc);

      }

Exit_Scroll:
      iSem--;
   }
}

// WM_SIZE
VOID Do_WM_SIZE( HWND hWnd, LPARAM lParam )
{
   g_wm_size.cx = LOWORD(lParam);
   g_wm_size.cy = HIWORD(lParam);   // width and height

   SetScrollRanges(hWnd, LOWORD(lParam), HIWORD(lParam));

   UpdateWP( hWnd );

   Do_ResizeStatus( hWnd );

}

VOID Do_WM_KEYDOWN( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
   /* Translate keyboard messages to scroll commands */
   switch (wParam)
   {
   case VK_UP:
        PostMessage (hWnd, WM_VSCROLL, SB_LINEUP,   0L);
        break;

    case VK_DOWN:
        PostMessage (hWnd, WM_VSCROLL, SB_LINEDOWN, 0L);
        break;

    case VK_PRIOR:
        PostMessage (hWnd, WM_VSCROLL, SB_PAGEUP,   0L);
        break;

    case VK_NEXT:
        PostMessage (hWnd, WM_VSCROLL, SB_PAGEDOWN, 0L);
        break;

    case VK_HOME:
        PostMessage (hWnd, WM_HSCROLL, SB_PAGEUP,   0L);
        break;

    case VK_END:
        PostMessage (hWnd, WM_HSCROLL, SB_PAGEDOWN, 0L);
        break;

    case VK_LEFT:
        PostMessage (hWnd, WM_HSCROLL, SB_LINEUP,   0L);
        break;

    case VK_RIGHT:
        PostMessage (hWnd, WM_HSCROLL, SB_LINEDOWN, 0L);
        break;
#ifdef   ADDMAG2
    case VK_SHIFT:
       bShiftDown = TRUE;
       break;
#endif   // ADDMAG2
   }
}

VOID Do_WM_KEYUP( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
   switch (wParam)
   {
   case VK_UP:
   case VK_DOWN:
   case VK_PRIOR:
   case VK_NEXT:
      PostMessage (hWnd, WM_VSCROLL, SB_ENDSCROLL, 0L);
      break;

   case VK_HOME:
   case VK_END:
   case VK_LEFT:
   case VK_RIGHT:
      PostMessage (hWnd, WM_HSCROLL, SB_ENDSCROLL, 0L);
      break;
#ifdef   ADDMAG2
   case VK_SHIFT:
       bShiftDown = FALSE;
       break;
#endif   // ADDMAG2
   }
}

VOID Do_WM_VSCROLL( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
   INT   iMax, iMin, iPos, dn;
   RECT  rc;

   /* Calculate new vertical scroll position */
   GetScrollRange (hWnd, SB_VERT, &iMin, &iMax);
   iPos = GetScrollPos (hWnd, SB_VERT);
   GetClientRect (hWnd, &rc);

   switch (GET_WM_VSCROLL_CODE(wParam, lParam))
   {
   case SB_LINEDOWN:
      dn =  rc.bottom / 16 + 1;
      break;
   case SB_LINEUP:
      dn = -rc.bottom / 16 + 1;
      break;
   case SB_PAGEDOWN:
      dn =  rc.bottom / 2  + 1;
      break;
   case SB_PAGEUP:
      dn = -rc.bottom / 2  + 1;
      break;
   case SB_THUMBTRACK:
   case SB_THUMBPOSITION:
      dn = GET_WM_VSCROLL_POS(wParam, lParam)-iPos;
      break;
   default:
      dn = 0;
      break;
   }

   /* Limit scrolling to current scroll range */
   if (dn = BOUND(iPos + dn, iMin, iMax) - iPos) {
      ScrollWindow (hWnd, 0, -dn, NULL, NULL);
      SetScrollPos (hWnd, SB_VERT, iPos + dn, TRUE);
   }
}

VOID Do_WM_HSCROLL( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
   INT   iMax, iMin, iPos, dn;
   RECT  rc;
   /* Calculate new horizontal scroll position */
   GetScrollRange (hWnd, SB_HORZ, &iMin, &iMax);
   iPos = GetScrollPos (hWnd, SB_HORZ);
   GetClientRect (hWnd, &rc);

   switch (GET_WM_HSCROLL_CODE(wParam, lParam))
   {
   case SB_LINEDOWN:
      dn =  rc.right / 16 + 1;
      break;
   case SB_LINEUP:
      dn = -rc.right / 16 + 1;
      break;
   case SB_PAGEDOWN:
      dn =  rc.right / 2  + 1;
      break;
   case SB_PAGEUP:
      dn = -rc.right / 2  + 1;
      break;
   case SB_THUMBTRACK:
   case SB_THUMBPOSITION:
      dn = GET_WM_HSCROLL_POS(wParam, lParam)-iPos;
      break;
   default:
      dn = 0;
      break;
   }
   /* Limit scrolling to current scroll range */
   if (dn = BOUND (iPos + dn, iMin, iMax) - iPos) {
      ScrollWindow (hWnd, -dn, 0, NULL, NULL);
      SetScrollPos (hWnd, SB_HORZ, iPos + dn, TRUE);
   }
}

#if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
/* =======
wParam
The high-order word indicates the distance the wheel is rotated, 
expressed in multiples or divisions of WHEEL_DELTA, which is 120. 

A positive value indicates that the wheel was rotated forward, away 
from the user; a negative value indicates that the wheel was rotated 
backward, toward the user. 

The low-order word indicates whether various virtual keys are down. 
This parameter can be one or more of the following values. 
MK_CONTROL = The CTRL key is down.
MK_LBUTTON = The left mouse button is down.
MK_MBUTTON = The middle mouse button is down.
MK_RBUTTON = The right mouse button is down.
MK_SHIFT   = The SHIFT key is down.
MK_XBUTTON1 = Windows 2000/XP: The first X button is down.
MK_XBUTTON2 = Windows 2000/XP: The second X button is down.
lParam
The low-order word specifies the x-coordinate of the pointer,
 relative to the upper-left corner of the screen. 
The high-order word specifies the y-coordinate of the pointer,
 relative to the upper-left corner of the screen. 
   ======= */
VOID Do_WM_MOUSEWHEEL( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
   int fwKeys = GET_KEYSTATE_WPARAM(wParam);
   int zDelta = GET_WHEEL_DELTA_WPARAM(wParam);
   int xPos = GET_X_LPARAM(lParam); 
   int yPos = GET_Y_LPARAM(lParam); 
   WPARAM Direction;
   Direction = (zDelta < 0) ? SB_LINEDOWN : SB_LINEUP;
    //gtab_msg_vscroll(hwnd, ptab,
    //((HIWORD(wParam) & 0x8000) ? SB_LINEDOWN : SB_LINEUP), 0 );
   PostMessage (hWnd, WM_VSCROLL, Direction, 0L);

}
#endif   // #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)

