// Timer.cxx

#include "testap3.h"
#include <math.h>
#include <time.h>
#include <strsafe.h>

#define  USE_WIN32_TIME

UINT_PTR msElapsed = 0;
UINT_PTR uiTimer1 = 0;
UINT_PTR TickCount = 0;

static long frames32 = 0;
static long long frames64 = 0;

static time_t last_time = 0;
TCHAR szFrameRate[256] = { L"\0" };
typedef struct tagMYTM {
   int tm_seconds;
   int tm_usec;
}MYTM, * PMYTM;

//#define  FRAME_RATE_HISTORY   64
//#define  FRAME_RATE_HISTORY   16
#define  FRAME_RATE_HISTORY   8

double lastavelapsed = 0.0;
double dElapsedHist[FRAME_RATE_HISTORY];
long long lFramespersec[FRAME_RATE_HISTORY];

#ifdef   USE_WIN32_TIME

static BOOL _s_bGotFreq = FALSE;
static LARGE_INTEGER _s_freq;
static BOOL  _s_bGotCnt;
static LARGE_INTEGER _s_count;
void Init_Win_Timer( void )
{
   _s_bGotFreq = QueryPerformanceFrequency( &_s_freq ); // signal have high performance timer
   // and get frequency, in counts per second
   if(_s_bGotFreq) {
      _s_bGotCnt = QueryPerformanceCounter( &_s_count ); // LARGE_INTEGER *lpPerformanceCount
      if( _s_bGotCnt ) {
         sprtf( "PerformanceCounter: freq = %I64d, count = %I64d\n",
            _s_freq, _s_count );
      } else {
         _s_count.QuadPart = GetTickCount();
         sprtf( "PerformanceCounter: FAILED, tickcount = %I64d\n",
            _s_count );
      }
   }
}

void Get_Win32_Counter( LARGE_INTEGER * pli )
{
   if(_s_bGotFreq && _s_bGotCnt)
      QueryPerformanceCounter( pli );  // get system count
   else
      pli->QuadPart = GetTickCount();
}

void Set_Win32_Stamp( PMYTM pmt )
{
   LARGE_INTEGER li;
   Get_Win32_Counter( &li );  // get NEW count
   if(_s_bGotFreq && _s_bGotCnt) {
      double cps = (double)li.QuadPart / (double)_s_freq.QuadPart;
      pmt->tm_seconds = (long)cps;
      pmt->tm_usec = (long)( cps - (double)( pmt->tm_seconds * 1000000 ) ); // microseconds
   } else {
      pmt->tm_seconds = (long)(li.QuadPart / 1000);     // seconds, and 
      pmt->tm_usec = (long)(( li.QuadPart - ( pmt->tm_seconds * 1000 ) ) * 1000); // microseconds
   }
}
#endif // USE_WIN32_TIME


PTSTR GetFrameRate( void )
{
   return szFrameRate;
}

// difference between time stamps in microseconds (usec)
long Get_Time_Diff_us( PMYTM a, PMYTM b)
{
    long lg = (1000000 * (a->tm_seconds - b->tm_seconds)) + (a->tm_usec - b->tm_usec);
    return lg;
}

void Set_Time_Stamp( PMYTM pmt ) {
#ifdef USE_WIN32_TIME
   Set_Win32_Stamp( pmt );
#else // !USE_WIN32_TIME
    unsigned int t = timeGetTime(); // Returns the system time, in milliseconds
    pmt->tm_seconds = t / 1000;     // seconds, and 
    pmt->tm_usec = ( t - ( pmt->tm_seconds * 1000 ) ) * 1000; // microseconds
#endif   // USE_WIN32_TIME y/n
}

// This function returns the number of microseconds since the last
// time it was called.
//int fgGetTimeInterval( void ) {
long Get_Elapsed_Time_us( void )
{
   int interval;
   static int inited = 0;
   static MYTM last;
   MYTM current;
   if ( !inited ) {
      int i;
      inited = 1;
#ifdef USE_WIN32_TIME
      Init_Win_Timer();   
#endif // USE_WIN32_TIME
       Set_Time_Stamp( &last );
       interval = 0;
       for( i = 0; i < FRAME_RATE_HISTORY; i++ ) {
          dElapsedHist[i] = 0.0;
          lFramespersec[i] = 0;
       }
    } else {
       Set_Time_Stamp( &current );
       interval = Get_Time_Diff_us( &current, &last );
       last = current;
    }
    return interval;
}

void Do_Frame_Rate(void)
{
   static int waitfor = 0;
   static int waitcnt = 0;
   long elapsed = Get_Elapsed_Time_us();
   double tmp = lastavelapsed;
   int i;
   if( elapsed > 0.0 ) {
      double accum = 0.0;
      for( i = FRAME_RATE_HISTORY - 2; i >= 0; i-- ) {
         tmp = dElapsedHist[i];
         accum += tmp;
         dElapsedHist[i+1] = tmp;
      }
      // tmp = 1000000.0 / (double)elapsed;
      tmp = (double)elapsed;
      dElapsedHist[0] = tmp;
      accum += tmp;

      tmp = (accum / (double)FRAME_RATE_HISTORY);
      lastavelapsed = tmp;
   }

   time_t new_time = time(NULL);
   if( new_time != last_time )
   {
      // each SECOND
      long long total, cnt;
      total = 0;
      // move each up one slot
      for( i = FRAME_RATE_HISTORY - 2; i >= 0; i-- ) {
         cnt = lFramespersec[i];
         total += cnt;
         lFramespersec[i+1] = cnt;
      }
      lFramespersec[0] = frames32;  // add in NEW count
      total += frames32;      // also to TOTAL

      if(waitfor) {
         frames64 = (total / FRAME_RATE_HISTORY);  // get AVERAGE fps
         double tmp2 = 1000000.0 / (double)frames64;
         StringCbPrintf(szFrameRate, 256, L"av. %I64d fps, elap %0.3f (%0.3f)", frames64, tmp, tmp2 );
      } else {
         StringCbPrintf(szFrameRate, 256, L"%d of %d", waitcnt, FRAME_RATE_HISTORY );
      }
      frames32 = 0;  // restart per-second counter
      last_time = new_time;   // update to this count
      waitcnt++;
      if( !waitfor && (waitcnt >= FRAME_RATE_HISTORY) )
         waitfor = 1;
   }
   frames32++;   // frames per second
}

void Do_SecondTimer( HWND hWnd )
{
   if( m_bAnim && m_hBitmap && m_hDIB && gActionAnim ) {
      // we have something to animate
      gActionAnim(hWnd);
      PaintStatusMousePos(hWnd);
   }
}

void Do_WM_TIMER( HWND hWnd )
{
   TickCount++;
   if(TickCount == 1) {
      if( gDoAuto && !IsFileListEmpty() ) {
         //SetRect( &m_Clip, // rectangle
         //   30, // left side
         //   20, // top side
         //   230, // right side
         //   140 ); // bottom side
         //PostMessage( hWnd, WM_COMMAND, IDM_ACTIONTEST, 0 );
         PostMessage( hWnd, WM_COMMAND, IDM_FILELIST, 0 );
      }
   }
   msElapsed += TIMER_INTERVAL1;
   if(msElapsed >= 1000) {
      Do_SecondTimer( hWnd );
      msElapsed = 0;
   }

   Do_StatusTimer();
}

// eof - Timer.cxx
